@@ -1,80 +1,105 @@
-"""Definitions of fields."""
-
 import datetime
 
 import six
 from dateutil.parser import parse
 
-from .error import ValidationError
+from .errors import ValidationError
+from .collections import ModelCollection
 
 
 class BaseField(object):
 
     """Base class for all fields."""
 
-    _types = None
+    types = None
 
     def __init__(
             self,
             required=False,
-            data_transformer=None,
             help_text=None,
             validators=None):
-        """Init."""
+        self._memory = {}
         self.required = required
-        self.data_transformer = data_transformer
         self.help_text = help_text
+        self._assign_validators(validators)
 
+    def _assign_validators(self, validators):
         if validators and not isinstance(validators, list):
             validators = [validators]
         self.validators = validators or []
 
-    @property
-    def types(self):
-        """Get types."""
-        return self._types
+    def __set__(self, obj, value):
+        value = self.parse_value(value)
+        self.validate(value)
+        self._memory[obj] = value
 
-    def validate(self, name, value):
-        """Validate value."""
-        if self._types is None:
-            raise ValidationError(
-                'Field "{}" is of type "{}" that is not usable, try '
-                'different field type.'.format(name, type(self).__name__))
+    def __get__(self, obj, owner=None):
+        if obj is None:
+            return self
 
-        if value is not None and not isinstance(value, self._types):
-            raise ValidationError(
-                'Value of field "{}" is wrong, expected type "{}"'.format(
-                    name,
-                    ', '.join([t.__name__ for t in self._types])
-                ))
+        self._check_value(obj)
+        return self._memory[obj]
 
-        if value is None and self.required:
-            raise ValidationError('Field "{}" is required!'.format(name))
+    def _check_value(self, obj):
+        if obj not in self._memory:
+            self.__set__(obj, self.get_default_value())
 
+    def validate_for_object(self, obj):
+        value = self.__get__(obj)
+        self.validate(value)
+
+    def validate(self, value):
+        self._check_types()
+        self._validate_against_types(value)
+        self._check_against_required(value)
         self._validate_with_custom_validators(value)
 
+    def _check_against_required(self, value):
+        if value is None and self.required:
+            raise ValidationError('Field is required!')
+
+    def _validate_against_types(self, value):
+        if value is not None and not isinstance(value, self.types):
+            raise ValidationError(
+                'Value is wrong, expected type "{}"'.format(
+                    ', '.join([t.__name__ for t in self.types])
+                ),
+                value
+            )
+
+    def _check_types(self):
+        if self.types is None:
+            raise ValidationError(
+                'Field "{}" is not usable, try '
+                'different field type.'.format(type(self).__name__))
+
     def to_struct(self, value):
         """Cast value to Python structure."""
         return value
 
     def parse_value(self, value):
-        """Parse value from primitive to desired format."""
-        if self.data_transformer:
-            return self.data_transformer.transform(value)
-        else:
-            return value
+        """Parse value from primitive to desired format.
+
+        Each field can parse value to form it wants it to be (like string or
+        int).
+
+        """
+        return value
 
     def _validate_with_custom_validators(self, value):
-        if self.validators:
-            for validator in self.validators:
-                try:
-                    validator.validate(value)
-                except AttributeError:
-                    validator(value)
+        for validator in self.validators:
+            try:
+                validator.validate(value)
+            except AttributeError:
+                validator(value)
 
     @staticmethod
-    def get_value_replacement():
-        """Get replacement for field."""
+    def get_default_value():
+        """Get default value for field.
+
+        Each field can specify its default.
+
+        """
         return None
 
 
@@ -82,28 +107,28 @@ class StringField(BaseField):
 
     """String field."""
 
-    _types = six.string_types
+    types = six.string_types
 
 
 class IntField(BaseField):
 
     """Integer field."""
 
-    _types = (int,)
+    types = (int,)
 
 
 class FloatField(BaseField):
 
     """Float field."""
 
-    _types = (float, int)
+    types = (float, int)
 
 
 class BoolField(BaseField):
 
     """Bool field."""
 
-    _types = (bool,)
+    types = (bool,)
 
     def parse_value(self, value):
         """Cast value to `bool`."""
@@ -115,12 +140,7 @@ class ListField(BaseField):
 
     """List field."""
 
-    _types = (list,)
-
-    @property
-    def items_types(self):
-        """Get items types."""
-        return self._items_types
+    types = (list,)
 
     def __init__(self, items_types=None, *args, **kwargs):
         """Init.
@@ -129,71 +149,90 @@ class ListField(BaseField):
         of items use validators.
 
         """
-        if items_types:
-            try:
-                self._items_types = tuple(items_types)
-            except TypeError:
-                self._items_types = items_types,
-        else:
-            self._items_types = tuple()
+        self._assign_types(items_types)
         super(ListField, self).__init__(*args, **kwargs)
         self.required = False
 
-    def validate(self, name, value):
-        """Validation."""
-        super(ListField, self).validate(name, value)
+    def _assign_types(self, items_types):
+        if items_types:
+            try:
+                self.items_types = tuple(items_types)
+            except TypeError:
+                self.items_types = items_types,
+        else:
+            self.items_types = tuple()
 
-        if len(self._items_types) == 0:
+    def validate(self, value):
+        super(ListField, self).validate(value)
+
+        if len(self.items_types) == 0:
             return
 
         try:
             for item in value:
-                if not isinstance(item, self._items_types):
-                    raise ValidationError(
-                        'All items of "{}" must be instances '
-                        'of "{}", and not "{}".'.format(
-                            name,
-                            ', '.join([t.__name__ for t in self._items_types]),
-                            type(item).__name__
-                        ))
+                self.validate_single_value(item)
         except TypeError:
             pass
 
-    def to_struct(self, value):
-        """Cast value to structure."""
-        result = []
-        for item in value:
-            result.append(item.to_struct())
-        return result
+    def validate_single_value(self, item):
+        if len(self.items_types) == 0:
+            return
 
-    @staticmethod
-    def get_value_replacement():
-        """Get replacement for field."""
-        return list()
+        if not isinstance(item, self.items_types):
+            raise ValidationError(
+                'All items must be instances '
+                'of "{}", and not "{}".'.format(
+                    ', '.join([t.__name__ for t in self.items_types]),
+                    type(item).__name__
+                ))
+
+    def to_struct(self, value):
+        """Cast value to list."""
+        return [item.to_struct() for item in value]
+
+    def get_default_value(self):
+        return ModelCollection(self)
 
     def parse_value(self, values):
-        """Parse value to proper type."""
-        embed_type = self._items_types[0]
+        """Cast value to proper collection."""
+        result = self.get_default_value()
+
+        if not values:
+            return result
+
+        if not isinstance(values, list):
+            return values
+
+        embed_type = self.items_types[0]
 
         if not hasattr(getattr(embed_type, 'populate', None), '__call__'):
             return values
 
-        if len(self._items_types) != 1:
+        self._check_items_types_count()
+        self._parse_values_to_result(values, embed_type, result)
+
+        return result
+
+    def _check_items_types_count(self):
+        if len(self.items_types) != 1:
             raise ValidationError(
                 'Cannot decide which type to choose from "{}".'.format(
-                    ', '.join([t.__name__ for t in self._items_types])
+                    ', '.join([t.__name__ for t in self.items_types])
                 )
             )
 
-        result = self.get_value_replacement()
-
+    def _parse_values_to_result(self, values, embed_type, result):
         try:
             for value in values:
-                result.append(embed_type(**value))
+                self._append_value_to_result(value, result, embed_type)
         except TypeError:
             raise ValidationError('Given value for field is not iterable.')
 
-        return result
+    def _append_value_to_result(self, value, result, embed_type):
+        if isinstance(value, self.items_types):
+            result.append(value)
+        else:
+            result.append(embed_type(**value))
 
 
 class EmbeddedField(BaseField):
@@ -201,44 +240,46 @@ class EmbeddedField(BaseField):
     """Field for embedded models."""
 
     def __init__(self, model_types, *args, **kwargs):
-        """Init."""
-        try:
-            iter(model_types)
-            self._types = tuple(model_types)
-        except TypeError:
-            self._types = (model_types,)
-
+        self._assign_model_types(model_types)
         super(EmbeddedField, self).__init__(*args, **kwargs)
 
-    def validate(self, name, value):
-        """Validation."""
-        super(EmbeddedField, self).validate(name, value)
+    def _assign_model_types(self, model_types):
+        try:
+            iter(model_types)
+            self.types = tuple(model_types)
+        except TypeError:
+            self.types = (model_types,)
+
+    def validate(self, value):
+        super(EmbeddedField, self).validate(value)
         try:
             value.validate()
         except AttributeError:
             pass
 
     def parse_value(self, value):
-        """Parse value to proper type."""
+        """Parse value to proper model type."""
         if not isinstance(value, dict):
             return value
 
-        if len(self._types) != 1:
+        embed_type = self._get_embed_type()
+        return embed_type(**value)
+
+    def _get_embed_type(self):
+        if len(self.types) != 1:
             raise ValidationError(
                 'Cannot decide which type to choose from "{}".'.format(
-                    ', '.join([t.__name__ for t in self._types])
+                    ', '.join([t.__name__ for t in self.types])
                 )
             )
-        embed_type = self._types[0]
-
-        return embed_type(**value)
+        return self.types[0]
 
 
 class TimeField(StringField):
 
     """Time field."""
 
-    _types = (datetime.time,)
+    types = (datetime.time,)
 
     def __init__(self, str_format=None, *args, **kwargs):
         """Init.
@@ -258,6 +299,8 @@ class TimeField(StringField):
 
     def parse_value(self, value):
         """Parse string into instance of `time`."""
+        if isinstance(value, datetime.time):
+            return value
         return parse(value).timetz()
 
 
@@ -265,7 +308,7 @@ class DateField(StringField):
 
     """Date field."""
 
-    _types = (datetime.date,)
+    types = (datetime.date,)
     default_format = '%Y-%m-%d'
 
     def __init__(self, str_format=None, *args, **kwargs):
@@ -286,6 +329,8 @@ class DateField(StringField):
 
     def parse_value(self, value):
         """Parse string into instance of `date`."""
+        if isinstance(value, datetime.date):
+            return value
         return parse(value).date()
 
 
@@ -293,7 +338,7 @@ class DateTimeField(StringField):
 
     """Datetime field."""
 
-    _types = (datetime.datetime,)
+    types = (datetime.datetime,)
 
     def __init__(self, str_format=None, *args, **kwargs):
         """Init.
@@ -313,4 +358,6 @@ class DateTimeField(StringField):
 
     def parse_value(self, value):
         """Parse string into instance of `datetime`."""
+        if isinstance(value, datetime.datetime):
+            return value
         return parse(value)
